@@ -1,4 +1,5 @@
 import { DragDropContext, Droppable } from '@hello-pangea/dnd';
+import { useCallback, useMemo } from 'react';
 import { useFormContext, useWatch } from 'react-hook-form';
 
 import {
@@ -8,13 +9,18 @@ import {
   getNextRefId,
   rangeUtil,
   LoadingState,
+  PanelData,
+  getDefaultTimeRange,
 } from '@grafana/data';
 import { t, Trans } from '@grafana/i18n';
 import { getDataSourceSrv } from '@grafana/runtime';
 import { DataQuery } from '@grafana/schema';
 import { Button, Stack, Text, Box, Alert } from '@grafana/ui';
+import { VizWrapper } from 'app/features/alerting/unified/components/rule-editor/VizWrapper';
+import { useAlertQueryRunner } from 'app/features/alerting/unified/components/rule-editor/query-and-alert-condition/useAlertQueryRunner';
 import { getDefaultOrFirstCompatibleDataSource } from 'app/features/alerting/unified/utils/datasource';
 import { QueryEditorRow } from 'app/features/query/components/QueryEditorRow';
+import { AlertQuery } from 'app/types/unified-alerting-dto';
 
 import { AlertEnrichmentFormData } from './form';
 
@@ -25,7 +31,12 @@ interface EnricherConfigProps {
 export function QueryEnricherConfig({ stepPath }: EnricherConfigProps) {
   const { setValue, control } = useFormContext<AlertEnrichmentFormData>();
 
-  const queries = useWatch({ control, name: `${stepPath}.enricher.dataSource.raw.request.queries` }) || [];
+  const watchedQueries = useWatch({ control, name: `${stepPath}.enricher.dataSource.raw.request.queries` });
+
+  // Memoize queries to prevent unnecessary re-renders
+  const queries = useMemo(() => watchedQueries || [], [watchedQueries]);
+
+  const { queryPreviewData, runQueries } = useAlertQueryRunner();
 
   /**
    * Helper function to update queries with the raw dataSource type.
@@ -40,6 +51,31 @@ export function QueryEnricherConfig({ stepPath }: EnricherConfigProps) {
     });
   };
 
+  /**
+   * Convert DataQuery to AlertQuery format for preview
+   */
+  const convertToAlertQueries = useCallback((dataQueries: DataQuery[]): AlertQuery[] => {
+    return dataQueries.map((query) => ({
+      refId: query.refId,
+      queryType: '',
+      model: query,
+      datasourceUid: query.datasource?.uid ?? '',
+      relativeTimeRange: getDefaultRelativeTimeRange(),
+    }));
+  }, []);
+
+  /**
+   * Run queries preview
+   */
+  const handleRunQueries = useCallback(() => {
+    if (queries.length === 0) {
+      return;
+    }
+    const alertQueries = convertToAlertQueries(queries);
+    // For enrichment queries, we don't have a condition, so we pass an empty string
+    runQueries(alertQueries, '');
+  }, [queries, convertToAlertQueries, runQueries]);
+
   const handleAddQuery = () => {
     const defaultDataSource = getDefaultOrFirstCompatibleDataSource();
     const newQuery: DataQuery = {
@@ -136,25 +172,35 @@ export function QueryEnricherConfig({ stepPath }: EnricherConfigProps) {
                         );
                       }
 
+                      const queryData: PanelData = queryPreviewData[query.refId] ?? {
+                        state: LoadingState.NotStarted,
+                        series: [],
+                        timeRange: getDefaultTimeRange(),
+                      };
+
+                      const showVisualization = queryData.state !== LoadingState.NotStarted;
+
                       return (
-                        <QueryEditorRow
-                          key={query.refId}
-                          id={query.refId}
-                          index={index}
-                          data={{ state: LoadingState.Done, series: [], timeRange: range }}
-                          query={query}
-                          queries={queries}
-                          dataSource={dsSettings}
-                          onChangeDataSource={(settings) => handleDataSourceChange(settings, index)}
-                          onChange={(updatedQuery) => handleQueryChange(updatedQuery, index)}
-                          onRemoveQuery={handleRemoveQuery}
-                          onAddQuery={handleAddQuery}
-                          onRunQuery={() => {}} // Not needed for enrichment config
-                          range={range}
-                          app={CoreApp.UnifiedAlerting}
-                          collapsable={false}
-                          hideHideQueryButton={true}
-                        />
+                        <Stack key={query.refId} direction="column" gap={0.5}>
+                          <QueryEditorRow
+                            id={query.refId}
+                            index={index}
+                            data={queryData}
+                            query={query}
+                            queries={queries}
+                            dataSource={dsSettings}
+                            onChangeDataSource={(settings) => handleDataSourceChange(settings, index)}
+                            onChange={(updatedQuery) => handleQueryChange(updatedQuery, index)}
+                            onRemoveQuery={handleRemoveQuery}
+                            onAddQuery={handleAddQuery}
+                            onRunQuery={handleRunQueries}
+                            range={range}
+                            app={CoreApp.UnifiedAlerting}
+                            collapsable={false}
+                            hideHideQueryButton={true}
+                          />
+                          {showVisualization && <VizWrapper data={queryData} />}
+                        </Stack>
                       );
                     })}
                     {provided.placeholder}
